@@ -8,7 +8,7 @@
 #'
 #' @section Methods:
 #' \describe{
-#'  \item{\code{new(op, url, version, typeName, logger, ...)}}{
+#'  \item{\code{new(capabilities, op, url, version, typeName, logger, ...)}}{
 #'    This method is used to instantiate a WFSDescribeFeatureType object
 #'  }
 #' }
@@ -18,14 +18,16 @@
 #' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
 #'
 WFSDescribeFeatureType <- R6Class("WFSDescribeFeatureType",
-  inherit = OWSRequest,
+  inherit = OWSHttpRequest,
   private = list(
-    name = "DescribeFeatureType"
+    xmlElement = "DescribeFeatureType",
+    xmlNamespacePrefix = "WFS"
   ),
   public = list(
-    initialize = function(op, url, version, typeName, logger = NULL, ...) {
+    initialize = function(capabilities, op, url, version, typeName, logger = NULL, ...) {
       namedParams <- list(service = "WFS", version = version, typeName = typeName)
-      super$initialize(op, "GET", url, request = private$name,
+      super$initialize(element = private$xmlElement, namespacePrefix = private$namespacePrefix,
+                       capabilities, op, "GET", url, request = "DescribeFeatureType",
                        namedParams = namedParams, mimeType = "text/xml", logger = logger,
                        ...)
       self$execute()
